<?php
// $Id$

/**
 * @file
 * Miscellaneous private functions for Versus.
 */

/**
 * Display current competitions that can be voted on.
 *
 * @param $round
 *   The current round.
 * @return
 *   HTML showing the current competitions and their content types
 *   that can be voted on.
 */
function versus_voting_competitions($round) {

  // Fetch the active competitions & their content types.
  $result = db_query(db_rewrite_sql("
    SELECT n.title, c.field_competition_round_value AS round, ct.nid,
           ct.delta, ct.field_competition_content_types_value AS type
    FROM {node} n
    INNER JOIN {content_type_competition} c ON n.nid = c.nid
    INNER JOIN {content_field_competition_content_types} ct ON c.nid = ct.nid
    WHERE c.field_competition_round_value = %d
    ORDER BY ct.nid, ct.vid, ct.delta
  "), $round);

  // Stick the fetched data into an array.
  $competitions = array();
  while ($db_object = db_fetch_object($result)) {
    $nid = $db_object->nid;
    $competitions[$nid]['nid'] = $db_object->nid;
    $competitions[$nid]['title'] = $db_object->title;
    $competitions[$nid]['types'][$db_object->delta] = $db_object->type;
  }

  // Set some instructions.
  $instructions = t('Which competition and content type would you like to vote on?');

  // Return the themed data.
  return theme('versus_voting_competitions', $instructions, $competitions);
}

/**
 * Process a voting request for a particular content type.
 *
 * @param $type
 *   The content type to vote on.
 * @param $competition
 *   The competition that the voting should be restricted to.
 *   This is irrelevant when the Competition module is not enabled.
 * @return
 *   Any output from the processing.
 * @todo
 *   Instead of containing code for other modules, this function should gather
 *   all conditions from modules that implement hook_versus_conditions(). The
 *   hook should return an array of conditions as strings.
 */
function versus_vote_process_type($type, $competition = NULL) {
  global $user;

  // Set any conditions that nodes must meet in order to become candidates.
  // By default, there are no conditions, and all nodes of a particular
  // type are candidates for voting.
  $conditions = array("TRUE");

  // Add conditions for the competition module.
  if (module_exists('competition')) {
    module_load_include('inc', 'competition', 'competition.private');

    // Make sure that we're in the voting phase.
    if (competition_phase_get_current() != COMPETITION_PHASE_VOTING) {
      $conditions[] = "FALSE";
    }

    // Each node to be voted upon must belong to the current round.
    // We already check for SQL injection attempts, so we don't need to here.
    /*
    $conditions[] = "(
      SELECT (
        SELECT entry.field_competition_entry_comp_nid
        FROM {content_type_competition_entry} entry
        WHERE entry.nid = (
          SELECT r.parent_nid FROM {relativity} r WHERE r.nid = n.nid
        )
      ) = $competition
    )";
    */
  }

  // Marshal the conditions.
  $conditions = implode(" AND ", $conditions);

  // Request the list of node IDs for the required type.
  $result = db_query(db_rewrite_sql("
    SELECT n.nid, n.type FROM {node} n WHERE n.type = '%s' AND $conditions
    ORDER BY RAND()
  "), $type);

  // Fetch them.
  $nids = array();
  while ($db_object = db_fetch_object($result)) {
    $nids[] = (object) array(
      'nid' => $db_object->nid,
      'type' => $db_object->type,
    );
  }

  // Initialize some voting options.
  $left = NULL;
  $right = NULL;
  $done = FALSE;

  // If there's only one result, assign it to the left option.
  if (count($nids) == 1) {
    $left = $nids[0];
  }

  // There are more than two candidates, so we need to show a left & right
  // option for the user to vote on.
  if (count($nids) >= 2) {

    // Request the user's previous votes so we'll know what not to show
    // him/her again.  In Drupal 5, the Voting API doesn't provide a useful
    // function for returning all of the votes of a particular user.
    // Therefore, we need to go around the API and dig into the DB directly.
    // This won't be necessary in Drupal 6 because the Drupal 6 version of
    // the Voting API provides a much more general way of obtaining such
    // data through the two functions votingapi_select_votes() and
    // votingapi_select_results().  It appears as though the latter grabs
    // data from the cache while the former does not.  We can use one of
    // these in the Drupal 6 version of Versus, and do away with the
    // following query.
    $result = db_query("
      SELECT content_id AS loser, value AS winner FROM {votingapi_vote}
      WHERE (content_type = 'node')
      AND (value_type = 'option')
      AND (tag = 'versus')
      AND (uid = %d)
    ", $user->uid);

    // Marshal the data into an array.
    $votes = array();
    while ($db_object = db_fetch_object($result)) {
      $votes[] = (object) array(
        'loser'  => $db_object->loser,
        'winner' => $db_object->winner,
      );
    }

    // Cycle through each pair of nodes.
    foreach ($nids as $pos1 => $node1) {
      foreach ($nids as $pos2 => $node2) {

        // Don't compare nodes that have already been compared,
        // or nodes that are of different types.
        if (($pos2 <= $pos1) || ($node1->type != $node2->type)) {
          continue;
        }

        // Create a pair with the first node as the loser, and the second
        // as the winner.
        $order1 = (object) array(
          'loser' => $node1->nid,
          'winner' => $node2->nid,
        );

        // Create a pair with the second node as the loser, and the first
        // as the winner.
        $order2 = (object) array(
          'loser' => $node2->nid,
          'winner' => $node1->nid,
        );

        // Check to see if either of them are in the list of previous votes
        // for this user.  If either of them are, we need to skip this pair
        // and try to find another one.
        if (in_array($order1, $votes) || in_array($order2, $votes)) {
          continue;
        }
        else /* this pair hasn't been voted on yet */ {

          // Assign the current pair to the left and right options,
          // and then get off this roller coaster.
          $left = $node1;
          $right = $node2;
          break;
        }
      }
    }
  }

  // If the left and right options haven't been set, we can only assume
  // that the user has voted on all possible pairs.  Therefore, the user
  // is done voting in this category.
  if (!($left || $right)) {
    $done = TRUE;
  }

  // Return a ballot based on what we've discovered above.
  return versus_voting_ballot($type, count($nids), $done, $left, $right);
}

/**
 * Prepare the output for the voting ballot.
 *
 * @param $candidates
 *   The size of the candidate pool.
 * @param $done
 *   TRUE if the user has finished voting, FALSE otherwise.
 * @param $left
 *   The node representing the choice on the left.
 * @param $right
 *   The node representing the choice on the right.
 * @return
 *   The themed output of whatever's on the voting ballot.
 */
function versus_voting_ballot($type, $candidates, $done = FALSE, $left = NULL, $right = NULL) {

  // Check if there's nothing to vote on.
  if (!$candidates) {
    drupal_set_title(t('No voting for this content type'));
    return t('There are no candidates of this type to vote on.  Please select a different type of content for voting over at <a href="/versus/vote">the selection page</a>.');
  }

  // Check if we've only got one candidate.  It, clearly, is the winner,
  // so there's nothing to vote on.  However, we should say what the winner
  // is, and provide a link to it.
  if ($candidates < 2) {
    drupal_set_title(t('No voting for this content type'));
    return t('There is only one candidate of this type available, so it wins by default.  Take a look at it by clicking <a href="/node/@nid">here</a>.',
      array('@nid' => $left->nid));
  }

  // Check if the user is done voting.
  if ($done) {
    drupal_set_title(t('Thanks for voting!'));
    $output = t('You have already voting on all pairs of candidates.  If you haven\'t voted for all types of content, please do so from the <a href="/versus/vote">selection page</a>.');
    $output .= '<p>' . views_page('versus_standings', 'standings', $type) . '</p>';
    return $output;
  }

  // Get all of the information for each voting option.
  $left = node_load($left->nid);
  $right = node_load($right->nid);

  // Set the page title.
  drupal_set_title(t('@left vs. @right', array(
    '@left' => check_plain($left->title),
    '@right' => check_plain($right->title),
  )));

  return theme('versus_voting_ballot', $left, $right);
}